package com.hzs.scm.sale.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.hzs.common.domain.sale.ext.SaDeliverExt;
import com.hzs.common.domain.scm.sale.IcSaleoutH;
import com.hzs.common.domain.scm.sale.ext.IcSaleoutHExt;
import com.hzs.scm.pub.param.ScmApproveParam;
import com.hzs.scm.sale.param.SaleOutParam;
import com.hzs.system.sys.dto.ApprovalBusinessResultDTO;
import com.hzs.system.sys.dto.LoginUser;

import java.util.List;

/**
 * 销售出库单表头 服务类
 */
public interface IIcSaleoutHService extends IService<IcSaleoutH> {

    /*
     *  保存销售出库单
     **/
    void saveSaleOutBySaDeliver(SaDeliverExt deliverExt);

    /*
     *  退货保存销售退货单
     **/
    void saveBackSaleOutByParam(IcSaleoutHExt saleOutExt, SaleOutParam saleOutParam);

    /*
     *  查询会员端显示的采购入库单
     **/
    List<IcSaleoutHExt> queryDisSaleOutExtCondition(SaleOutParam saleOutParam);

    /*
     *  根据主键查询销售出库单
     **/
    IcSaleoutHExt querySaleOutExtById(SaleOutParam saleOutParam);

    /*
     *  根据订单主键查询销售出库单
     **/
    IcSaleoutHExt querySaleOutExtId(Long pkId, Integer orderStatus);

    /*
     *  查询会员端显示的盘点
     **/
    IcSaleoutHExt queryDisSaleOutExtById(Long pkId, Integer orderStatus);
}
